@@ -1,9 +1,10 @@
 pmwgs <- function(dadm, type, pars = NULL, prior = NULL,
                   nuisance = NULL, nuisance_non_hyper = NULL, ...) {
   if(is.data.frame(dadm)) dadm <- list(dadm)
+  if(is.null(pars)) pars <- names(sampled_pars(attr(dadm[[1]], "prior")))
+  if(is.null(prior)) prior <- attr(dadm[[1]], "prior")
   dadm <- extractDadms(dadm)
-  if(is.null(pars)) pars <- dadm$pars
-  if(is.null(prior)) prior <- dadm$prior
+
   dadm_list <-dadm$dadm_list
   # Storage for the samples.
   subjects <- sort(as.numeric(unique(dadm$subjects)))
@@ -12,10 +13,10 @@ pmwgs <- function(dadm, type, pars = NULL, prior = NULL,
 
   if(!is.null(nuisance_non_hyper)){
     is_nuisance <- is.element(seq_len(length(pars)), nuisance_non_hyper)
-    type <- "single"
+    nuis_type <- "single"
   } else if(!is.null(nuisance)) {
     is_nuisance <- is.element(seq_len(length(pars)), nuisance)
-    type <- "diagonal"
+    nuis_type <- "diagonal"
   } else{
     is_nuisance <- rep(F, length(pars))
   }
@@ -24,15 +25,15 @@ pmwgs <- function(dadm, type, pars = NULL, prior = NULL,
   sampler_nuis <- NULL
   if(any(is_nuisance)){
     sampler_nuis <- list(
-      samples = sample_store(dadm, pars, type, integrate = F,
+      samples = sample_store(dadm, pars, nuis_type, integrate = F,
                                                     is_nuisance = !is_nuisance, ...),
       n_subjects = length(subjects),
       n_pars = sum(is_nuisance),
       nuisance = rep(F, sum(is_nuisance)),
-      type = type
+      type = nuis_type
     )
-    if(type == "single") sampler_nuis$samples <- NULL
-    sampler_nuis <- add_info(sampler_nuis, prior$prior_nuis, type, ...)
+    if(nuis_type == "single") sampler_nuis$samples <- NULL
+    sampler_nuis <- add_info(sampler_nuis, prior$prior_nuis, nuis_type, ...)
   }
   samples <- sample_store(dadm, pars, type, is_nuisance = is_nuisance, ...)
   sampler <- list(
@@ -63,7 +64,7 @@ init <- function(pmwgs, start_mu = NULL, start_var = NULL,
     startpoints_nuis <- get_startpoints(pmwgs$sampler_nuis, start_mu = NULL, start_var = NULL, type = type_nuis)
     startpoints_comb <- merge_group_level(startpoints$tmu, startpoints_nuis$tmu,
                                           startpoints$tvar, startpoints_nuis$tvar,
-                                          pmwgs$nuisance)
+                                          pmwgs$nuisance, startpoints$subj_mu)
     pmwgs$sampler_nuis$samples <- fill_samples(samples = pmwgs$sampler_nuis$samples,
                                                                       group_level = startpoints_nuis,
                                                                       j = 1,
@@ -114,7 +115,7 @@ init <- function(pmwgs, start_mu = NULL, start_var = NULL,
 #' # Initialize chains, 4 cores per chain, and parallelizing across our 3 chains as well
 #' # so 4*3 cores used.
 #' DDMaE <- init_chains(DDMaE, start_mu = mu, start_var = var,
-#'                      cores_per_chain = 1, cores_for_chains = 1)
+#'                      cores_per_chain = 1, cores_for_chains = 1, particles = 10)
 #' # Afterwards we can just use fit
 #' # DDMaE <- fit(DDMaE, cores_per_chain = 4)
 #' }
@@ -236,7 +237,7 @@ run_stage <- function(pmwgs,
     pars <- pars_comb <- gibbs_step(pmwgs, pmwgs$samples$alpha[!nuisance,,j-1], pmwgs$type)
     if(any(nuisance)){
       pars_nuis <- gibbs_step(pmwgs$sampler_nuis, pmwgs$samples$alpha[nuisance,,j-1], pmwgs$sampler_nuis$type)
-      pars_comb <- merge_group_level(pars$tmu, pars_nuis$tmu, pars$tvar, pars_nuis$tvar, nuisance)
+      pars_comb <- merge_group_level(pars$tmu, pars_nuis$tmu, pars$tvar, pars_nuis$tvar, nuisance, pars$subj_mu)
       pars_comb$alpha <- pmwgs$samples$alpha[,,j-1]
       pmwgs$sampler_nuis$samples <- fill_samples(samples = pmwgs$sampler_nuis$samples,
                                                                         group_level = pars_nuis,
@@ -684,15 +685,18 @@ calc_ll_manager <- function(proposals, dadm, model, component = NULL){
   return(lls)
 }
 
-merge_group_level <- function(tmu, tmu_nuis, tvar, tvar_nuis, is_nuisance){
+merge_group_level <- function(tmu, tmu_nuis, tvar, tvar_nuis, is_nuisance, subj_mu){
   n_pars <- length(is_nuisance)
   tmu_out <- numeric(n_pars)
   tmu_out[!is_nuisance] <- tmu
   tmu_out[is_nuisance] <- tmu_nuis
   tvar_out <- matrix(0, nrow = n_pars, ncol = n_pars)
   tvar_out[!is_nuisance, !is_nuisance] <- tvar
-  tvar_out[is_nuisance, is_nuisance] <- tvar_nuis
-  return(list(tmu = tmu_out, tvar = tvar_out))
+
+  subj_mu_out <- matrix(NA, ncol = ncol(subj_mu), nrow = length(tmu_out))
+  subj_mu_out[is_nuisance,] <- do.call(cbind, rep(list(c(tmu_nuis)), ncol(subj_mu)))
+  subj_mu_out[!is_nuisance,] <- subj_mu
+  return(list(tmu = tmu_out, tvar = tvar_out, subj_mu = subj_mu_out))
 }
 
 
@@ -726,6 +730,7 @@ run_hyper <- function(type, data, prior = NULL, iter = 1000, n_chains =3, ...){
     )
     class(sampler) <- "pmwgs"
     sampler <- add_info(sampler, prior, type = type, ...)
+    sampler$type <- type
     startpoints <- get_startpoints(sampler, start_mu = NULL, start_var = NULL, type = type)
     sampler$samples <- fill_samples(samples = sampler$samples, group_level = startpoints, proposals = NULL,
                                     j = 1, n_pars = sampler$n_pars, type = type)
@@ -743,6 +748,8 @@ run_hyper <- function(type, data, prior = NULL, iter = 1000, n_chains =3, ...){
     }
     emc[[j]] <- sampler
   }
+  emc[[1]]$type <- type
   class(emc) <- "emc"
+  emc <- subset(emc, filter = 1)
   return(emc)
 }
